import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;

import java.time.Duration;

import static org.junit.jupiter.api.Assertions.*;

@DisplayName("Test Bank account class")
class BankAccountTest {
    @Test
    @DisplayName("Withdraw 500 successfully")
    void testWithdraw() {
        BankAccount bankAccount = BankAccount.builder()
                .balance(500.00)
                .minimumBalance(-1000.00).build();
        bankAccount.withDraw(300.00);
        assertEquals(200, bankAccount.getBalance());
    }

    @Test
    @DisplayName("Deposit 400 successfully")
    void testDeposit() {
        BankAccount bankAccount = BankAccount.builder()
                .balance(400.00)
                .minimumBalance(0.00).build();
        bankAccount.deposit(500);
        assertEquals(900.00, bankAccount.getBalance());
    }

    @Test
    @DisplayName("Withdraw will become negative")
    void testWithdrawNotStuckAtZero() {
        BankAccount bankAccount = BankAccount.builder()
                .balance(500.00)
                .minimumBalance(-1000.00).build();
        bankAccount.withDraw(800.00);
        assertNotEquals(0, bankAccount.getBalance());
    }

    @Test
    @DisplayName("Test activation account after creation")
    void testActive() {
        BankAccount bankAccount = BankAccount.builder()
                .balance(400.00)
                .minimumBalance(0.00)
                .active(true).build();
        assertTrue(bankAccount.getActive());
    }

    @Test
    @DisplayName("test holder name set")
    void testHolderNameSet() {
        BankAccount bankAccount = BankAccount.builder()
                .balance(400.00)
                .minimumBalance(0.00)
                .active(true).build();
        assertNull(bankAccount.getHolderName());
        bankAccount.setHolderName("Bhargava");
        assertNotNull(bankAccount.getHolderName());
    }

    @Test
    @DisplayName("test no withdraw below minimum")
    void testNoWithdrawBelowMinimum() {
        BankAccount bankAccount = BankAccount.builder()
                .balance(500.00)
                .minimumBalance(-1000.00).build();
        assertThrows(RuntimeException.class, () -> bankAccount.withDraw(2000.00));
    }

    @Test
    @DisplayName("test withdraw and deposit without exceptions")
    void testWithdrawAndDepositWithoutExceptions() {
        BankAccount bankAccount = BankAccount.builder()
                .balance(500.00)
                .minimumBalance(-1000.00).build();
        assertAll(() -> bankAccount.deposit(200),
                () -> bankAccount.withDraw(300.00));
    }

    @Test
    @DisplayName("test deposit timeout")
    void testDepositTimeout() {
        BankAccount bankAccount = BankAccount.builder()
                .balance(500.00)
                .minimumBalance(-1000.00).build();
        assertTimeout(Duration.ofMillis(6), () -> bankAccount.withDraw(250.00));
        assertEquals(0.33, Double.valueOf((1.00 / 3.00)), 0.0033333333333333);

    }
}